@@ -9,15 +9,15 @@ from typing import List
 
 from .api_key import ApiKey
 from .batch_prediction import BatchPrediction
-from .bucket_verification import BucketVerification
-from .bucket_verification_instructions import BucketVerificationInstructions
-from .bucket_verification_result import BucketVerificationResult
 from .schema import Schema
+from .database_connector import DatabaseConnector
 from .dataset import Dataset
 from .dataset_version import DatasetVersion
 from .deployment import Deployment
 from .deployment_auth_token import DeploymentAuthToken
-from .external_connection import ExternalConnection
+from .file_connector import FileConnector
+from .file_connector_instructions import FileConnectorInstructions
+from .file_connector_verification import FileConnectorVerification
 from .model import Model
 from .model_metrics import ModelMetrics
 from .model_upload import ModelUpload
@@ -46,7 +46,7 @@ class ApiException(Exception):
 
 
 class ApiClient():
-    client_version = '0.11.2'
+    client_version = '0.13.0'
 
     def __init__(self, api_key=None, server='https://abacus.ai'):
         self.api_key = api_key
@@ -134,6 +134,34 @@ class ApiClient():
         '''    '''
         return self._call_api('predict', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, **kwargs})
 
+    def add_user_to_organization(self, email: str):
+        '''Invites a user to your organization. This method will send the specified email address an invitation link to join your organization.    '''
+        return self._call_api('addUserToOrganization', 'POST', query_params={}, body={'email': email})
+
+    def list_api_keys(self):
+        '''Lists all of the API keys created by the current member in the user's organization.    '''
+        return self._call_api('listApiKeys', 'GET', query_params={}, parse_type=ApiKey)
+
+    def list_organization_users(self):
+        '''Retrieves a list of all users in the organization.    '''
+        return self._call_api('listOrganizationUsers', 'GET', query_params={})
+
+    def get_user(self):
+        '''Get the current user's information, such as their name, email, admin status, etc.    '''
+        return self._call_api('getUser', 'GET', query_params={})
+
+    def set_user_as_admin(self, email: str):
+        '''Sets the specified user as an Organization Administrator. You must be an Organization Administrator to use this method. An Organization Administrator manages billing and the organization's users.    '''
+        return self._call_api('setUserAsAdmin', 'POST', query_params={}, body={'email': email})
+
+    def delete_api_key(self, api_key_id: str):
+        '''Delete a specified API Key. You can use the List API Keys API to find the list API Key IDs.    '''
+        return self._call_api('deleteApiKey', 'DELETE', query_params={'apiKeyId': api_key_id})
+
+    def remove_user_from_organization(self, email: str):
+        '''Removes the specified user from the organization. You must be an Organization Administrator to use this method, or be removing yourself from the organization.    '''
+        return self._call_api('removeUserFromOrganization', 'DELETE', query_params={'email': email})
+
     def create_project(self, name: str, use_case: str):
         '''Creates a project with your specified project name and use case. Creating a project creates a container for all of the datasets and the models that are associated with a particular problem/project that you would like to work on. For example, if you want to create a model to detect fraud, you have to first create a project, upload the associated datasets, and then create one or more models to get predictions for your use case.    '''
         return self._call_api('createProject', 'GET', query_params={'name': name, 'useCase': use_case}, parse_type=Project)
@@ -163,7 +191,7 @@ class ApiClient():
         return self._call_api('getSchema', 'GET', query_params={'projectId': project_id, 'datasetId': dataset_id}, parse_type=Schema)
 
     def rename_project(self, project_id: str, name: str):
-        '''This method renames a project after it is created. To rename the project, specify its ID and new name. The call will not return anything.    '''
+        '''This method renames a project after it is created. To rename the project, specify its ID and new name.    '''
         return self._call_api('renameProject', 'PATCH', query_params={'projectId': project_id, 'name': name})
 
     def set_column_data_type(self, project_id: str, dataset_id: str, column: str, data_type: str):
@@ -218,21 +246,41 @@ class ApiClient():
         '''Creates a streaming dataset. Use a streaming dataset if your dataset is receiving information from multiple sources over an extended period of time.    '''
         return self._call_api('createStreamingDataset', 'POST', query_params={}, body={'name': name, 'projectId': project_id, 'datasetType': dataset_type}, parse_type=Dataset)
 
-    def get_data_connector_verification(self, bucket: str, write_permission: bool = False):
+    def get_file_connector_instructions(self, bucket: str, write_permission: bool = False):
         '''Retrieves verification information to create a data connector.    '''
-        return self._call_api('getDataConnectorVerification', 'GET', query_params={'bucket': bucket, 'writePermission': write_permission}, parse_type=BucketVerificationInstructions)
+        return self._call_api('getFileConnectorInstructions', 'GET', query_params={'bucket': bucket, 'writePermission': write_permission}, parse_type=FileConnectorInstructions)
 
-    def list_data_connector_verifications(self):
+    def list_database_connectors(self):
+        '''List database Connectors    '''
+        return self._call_api('listDatabaseConnectors', 'GET', query_params={}, parse_type=DatabaseConnector)
+
+    def list_file_connectors(self):
         '''Retrieves a list of all connected services in the organization and their current verification status.    '''
-        return self._call_api('listDataConnectorVerifications', 'GET', query_params={}, parse_type=BucketVerification)
+        return self._call_api('listFileConnectors', 'GET', query_params={}, parse_type=FileConnector)
 
-    def verify_data_connector(self, bucket: str):
-        '''Checks to see if Abacus.AI can access the data connector.    '''
-        return self._call_api('verifyDataConnector', 'POST', query_params={}, body={'bucket': bucket}, parse_type=BucketVerificationResult)
+    def list_database_connector_objects(self, database_connector_id: str):
+        '''Lists querable objects in the database connector.    '''
+        return self._call_api('listDatabaseConnectorObjects', 'GET', query_params={'databaseConnectorId': database_connector_id})
 
-    def remove_data_connector(self, bucket: str):
+    def get_database_connector_object_schema(self, database_connector_id: str, object_name: str = None):
+        '''Get the schema of an object in an database connector.    '''
+        return self._call_api('getDatabaseConnectorObjectSchema', 'GET', query_params={'databaseConnectorId': database_connector_id, 'objectName': object_name})
+
+    def verify_database_connector(self, database_connector_id: str):
+        '''Checks to see if Abacus.AI can access the database.    '''
+        return self._call_api('verifyDatabaseConnector', 'GET', query_params={'databaseConnectorId': database_connector_id})
+
+    def verify_file_connector(self, bucket: str):
+        '''Checks to see if Abacus.AI can access the bucket.    '''
+        return self._call_api('verifyFileConnector', 'POST', query_params={}, body={'bucket': bucket}, parse_type=FileConnectorVerification)
+
+    def remove_database_connector(self, database_connector_id: str):
+        '''Disconnect an database connector.    '''
+        return self._call_api('removeDatabaseConnector', 'DELETE', query_params={'databaseConnectorId': database_connector_id})
+
+    def remove_file_connector(self, bucket: str):
         '''Removes a connected service from the specified organization.    '''
-        return self._call_api('removeDataConnector', 'DELETE', query_params={'bucket': bucket})
+        return self._call_api('removeFileConnector', 'DELETE', query_params={'bucket': bucket})
 
     def create_streaming_token(self):
         '''Creates a streaming token for the specified project. Streaming tokens are used to authenticate requests to append data to streaming datasets.    '''
@@ -247,7 +295,7 @@ class ApiClient():
         return self._call_api('deleteStreamingToken', 'DELETE', query_params={'streamingToken': streaming_token})
 
     def list_uploads(self):
-        '''Lists all the details associated with the dataset file upload including the current status of the upload (complete or not). Refer to the return table below for a complete description of the method. Use this API method to ensure that your dataset is uploaded properly and all the details associated with it are correct. If everything looks fine then you are good to use the dataset you uploaded in parts for your batch predictins and/or model training.    '''
+        '''Lists all the details associated with the dataset file upload including the current status of the upload (complete or not). Use this API method to ensure that your dataset is uploaded properly and all the details associated with it are correct. If everything looks fine then you are good to use the dataset you uploaded in parts for your batch predictins and/or model training.    '''
         return self._call_api('listUploads', 'GET', query_params={}, parse_type=Upload)
 
     def describe_upload(self, upload_id: str):
@@ -322,9 +370,9 @@ class ApiClient():
         '''Deletes the specified model version. Note that models versions are not recoverable after they are deleted.    '''
         return self._call_api('deleteModelVersion', 'DELETE', query_params={'modelVersion': model_version})
 
-    def create_deployment(self, model_id: str, name: str = None, description: str = None, deployment_config: dict = None):
+    def create_deployment(self, model_id: str, name: str = None, description: str = None, calls_per_second: int = None):
         '''Creates a deployment with the specified name and description for the specified model.    '''
-        return self._call_api('createDeployment', 'POST', query_params={}, body={'modelId': model_id, 'name': name, 'description': description, 'deploymentConfig': deployment_config}, parse_type=Deployment)
+        return self._call_api('createDeployment', 'POST', query_params={}, body={'modelId': model_id, 'name': name, 'description': description, 'callsPerSecond': calls_per_second}, parse_type=Deployment)
 
     def create_deployment_token(self, project_id: str):
         '''Creates a deployment token for the specified project.    '''
@@ -394,29 +442,41 @@ class ApiClient():
         '''Returns a list of forecasts for a given entity under the specified project deployment. Note that the inputs to the deployed model will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'holiday_yn' mapped to mapping 'FUTURE' in our system).    '''
         return self._call_api('getForecast', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'futureData': future_data, 'numPredictions': num_predictions, 'predictionStart': prediction_start})
 
-    def get_recommendations(self, deployment_token: str, deployment_id: str, query_data: dict, num_items: int = 50, page: int = 1, include_filters: list = [], exclude_filters: list = [], exclude_item_ids: list = [], score_field: str = '', scaling_factors: list = []):
+    def get_recommendations(self, deployment_token: str, deployment_id: str, query_data: dict, num_items: int = 50, page: int = 1, exclude_item_ids: list = [], score_field: str = '', scaling_factors: list = [], restrict_items: list = [], exclude_items: list = []):
         '''Returns a list of recommendations for a given user under the specified project deployment. Note that the inputs to this method, wherever applicable, will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'time' mapped to mapping 'TIMESTAMP' in our system).    '''
-        return self._call_api('getRecommendations', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'numItems': num_items, 'page': page, 'includeFilters': include_filters, 'excludeFilters': exclude_filters, 'excludeItemIds': exclude_item_ids, 'scoreField': score_field, 'scalingFactors': scaling_factors})
+        return self._call_api('getRecommendations', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'numItems': num_items, 'page': page, 'excludeItemIds': exclude_item_ids, 'scoreField': score_field, 'scalingFactors': scaling_factors, 'restrictItems': restrict_items, 'excludeItems': exclude_items})
 
-    def get_personalized_ranking(self, deployment_token: str, deployment_id: str, query_data: dict, scaling_factors: list = []):
+    def get_personalized_ranking(self, deployment_token: str, deployment_id: str, query_data: dict, preserve_ranks: list = [], scaling_factors: list = []):
         '''Returns a list of items with personalized promotions on them for a given user under the specified project deployment. Note that the inputs to this method, wherever applicable, will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'item_code' mapped to mapping 'ITEM_ID' in our system).    '''
-        return self._call_api('getPersonalizedRanking', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'scalingFactors': scaling_factors})
+        return self._call_api('getPersonalizedRanking', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'preserveRanks': preserve_ranks, 'scalingFactors': scaling_factors})
 
     def get_ranked_items(self, deployment_token: str, deployment_id: str, query_data: dict, preserve_ranks: list = [], scaling_factors: list = []):
         '''Returns a list of re-ranked items for a selected user when a list of items is required to be reranked according to the user's preferences. Note that the inputs to this method, wherever applicable, will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'item_code' mapped to mapping 'ITEM_ID' in our system).    '''
         return self._call_api('getRankedItems', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'preserveRanks': preserve_ranks, 'scalingFactors': scaling_factors})
 
-    def get_related_items(self, deployment_token: str, deployment_id: str, query_data: dict, num_items: int = 50, page: int = 1, include_filters: list = [], exclude_filters: list = [], scaling_factors: list = []):
+    def get_related_items(self, deployment_token: str, deployment_id: str, query_data: dict, num_items: int = 50, page: int = 1, scaling_factors: list = [], restrict_items: list = [], exclude_items: list = []):
         '''Returns a list of related items for a given item under the specified project deployment. Note that the inputs to this method, wherever applicable, will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'item_code' mapped to mapping 'ITEM_ID' in our system).    '''
-        return self._call_api('getRelatedItems', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'numItems': num_items, 'page': page, 'includeFilters': include_filters, 'excludeFilters': exclude_filters, 'scalingFactors': scaling_factors})
+        return self._call_api('getRelatedItems', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'numItems': num_items, 'page': page, 'scalingFactors': scaling_factors, 'restrictItems': restrict_items, 'excludeItems': exclude_items})
 
-    def batch_predict(self, deployment_id: str, input_location: str = None, output_location: str = None, name: str = None, refresh_schedule: str = None, global_prediction_args: dict = None):
+    def batch_predict(self, deployment_id: str, input_location: str = None, output_location: str = None, name: str = None, refresh_schedule: str = None, global_prediction_args: dict = None, explanations: bool = False):
         '''Starts a batch prediction task with the specified deployment ID, input location, output location, and batch prediction job name.    '''
-        return self._call_api('batchPredict', 'POST', query_params={'deploymentId': deployment_id}, body={'inputLocation': input_location, 'outputLocation': output_location, 'name': name, 'refreshSchedule': refresh_schedule, 'globalPredictionArgs': global_prediction_args}, parse_type=BatchPrediction)
+        return self._call_api('batchPredict', 'POST', query_params={'deploymentId': deployment_id}, body={'inputLocation': input_location, 'outputLocation': output_location, 'name': name, 'refreshSchedule': refresh_schedule, 'globalPredictionArgs': global_prediction_args, 'explanations': explanations}, parse_type=BatchPrediction)
 
-    def batch_prediction_from_upload(self, deployment_id: str, name: str = None, global_prediction_args: dict = None):
+    def batch_predict_from_database_connector(self, deployment_id: str, database_connector_id: str, object_name: str, name: str = None, connection_mode: str = 'output', columns: str = None, query_arguments: str = None, prediction_output_columns: str = None, refresh_schedule: str = None, global_prediction_args: dict = None):
+        '''Starts a batch prediction task with the specified deployment ID, input location, output location, and batch prediction job name.    '''
+        return self._call_api('batchPredictFromDatabaseConnector', 'POST', query_params={'deploymentId': deployment_id}, body={'databaseConnectorId': database_connector_id, 'objectName': object_name, 'name': name, 'connectionMode': connection_mode, 'columns': columns, 'queryArguments': query_arguments, 'predictionOutputColumns': prediction_output_columns, 'refreshSchedule': refresh_schedule, 'globalPredictionArgs': global_prediction_args}, parse_type=BatchPrediction)
+
+    def batch_prediction_from_upload(self, deployment_id: str, name: str = None, global_prediction_args: dict = None, explanations: bool = False):
         '''Starts a batch prediction task with the specified deployment ID, file, and batch prediction job name.    '''
-        return self._call_api('batchPredictionFromUpload', 'POST', query_params={'deploymentId': deployment_id}, body={'name': name, 'globalPredictionArgs': global_prediction_args}, parse_type=BatchPrediction)
+        return self._call_api('batchPredictionFromUpload', 'POST', query_params={'deploymentId': deployment_id}, body={'name': name, 'globalPredictionArgs': global_prediction_args, 'explanations': explanations}, parse_type=Upload)
+
+    def get_batch_prediction_result(self, batch_prediction_id: str):
+        '''Returns a stream containing the batch prediction results    '''
+        return self._call_api('getBatchPredictionResult', 'GET', query_params={'batchPredictionId': batch_prediction_id}, streamable_response=True)
+
+    def get_batch_prediction_connector_errors(self, batch_prediction_id: str):
+        '''Returns a stream containing the batch prediction external connection error, if it exists    '''
+        return self._call_api('getBatchPredictionConnectorErrors', 'GET', query_params={'batchPredictionId': batch_prediction_id}, streamable_response=True)
 
     def list_batch_predictions(self, deployment_id: str):
         '''Retrieves a list for the batch prediction jobs for the specified deployment.    '''
@@ -427,41 +487,13 @@ class ApiClient():
         return self._call_api('describeBatchPrediction', 'GET', query_params={'batchPredictionId': batch_prediction_id}, parse_type=BatchPrediction)
 
     def add_user_item_interaction(self, streaming_token: str, dataset_id: str, timestamp: int, user_id: str, item_id: str, event_type: str, additional_attributes: dict):
-        '''Record data to streaming dataset.    '''
+        '''Adds a user-item interaction record (data row) to a streaming dataset.    '''
         return self._call_api('addUserItemInteraction', 'POST', query_params={'streamingToken': streaming_token, 'datasetId': dataset_id}, body={'timestamp': timestamp, 'userId': user_id, 'itemId': item_id, 'eventType': event_type, 'additionalAttributes': additional_attributes})
 
     def upsert_user_attributes(self, streaming_token: str, dataset_id: str, user_id: str, user_attributes: dict):
-        '''Record attributes to streaming dataset.    '''
+        '''Adds a user attributes record (data row) to a streaming dataset.    '''
         return self._call_api('upsertUserAttributes', 'POST', query_params={'streamingToken': streaming_token, 'datasetId': dataset_id}, body={'userId': user_id, 'userAttributes': user_attributes})
 
     def upsert_item_attributes(self, streaming_token: str, dataset_id: str, item_id: str, item_attributes: dict):
-        '''Record attributes to streaming dataset.    '''
+        '''Adds an item attributes record (data row) to a streaming dataset.    '''
         return self._call_api('upsertItemAttributes', 'POST', query_params={'streamingToken': streaming_token, 'datasetId': dataset_id}, body={'itemId': item_id, 'itemAttributes': item_attributes})
-
-    def add_user_to_organization(self, email: str):
-        '''Invites a user to your organization. This method will send the specified email address an invitation link to join your organization.    '''
-        return self._call_api('addUserToOrganization', 'POST', query_params={}, body={'email': email})
-
-    def list_api_keys(self):
-        '''Lists all of the API keys created by the current member in the user's organization.    '''
-        return self._call_api('listApiKeys', 'GET', query_params={}, parse_type=ApiKey)
-
-    def list_organization_users(self):
-        '''Retrieves a list of all users in the organization.    '''
-        return self._call_api('listOrganizationUsers', 'GET', query_params={})
-
-    def get_user(self):
-        '''Get the current user's information, such as their name, email, admin status, etc.    '''
-        return self._call_api('getUser', 'GET', query_params={})
-
-    def set_user_as_admin(self, email: str):
-        '''Sets the specified user as an Organization Administrator. You must be an Organization Administrator to use this method. An Organization Administrator manages billing and the organization's users.    '''
-        return self._call_api('setUserAsAdmin', 'POST', query_params={}, body={'email': email})
-
-    def delete_api_key(self, api_key_id: str):
-        '''Delete a specified API Key. You can use the List API Keys API to find the list API Key IDs.    '''
-        return self._call_api('deleteApiKey', 'DELETE', query_params={'apiKeyId': api_key_id})
-
-    def remove_user_from_organization(self, email: str):
-        '''Removes the specified user from the organization. You must be an Organization Administrator to use this method, or be removing yourself from the organization.    '''
-        return self._call_api('removeUserFromOrganization', 'DELETE', query_params={'email': email})
