@@ -58,8 +58,8 @@ class Dataset():
     def wait_for_import(self, timeout=900):
         return self.client._poll(self, {'PENDING', 'IMPORTING'}, timeout=timeout)
 
-    def wait_for_inspection(self, timeout=900):
-        return self.client._poll(self, {'PENDING', 'UPLOADING', 'IMPORTING', 'INSPECTING'}, timeout=timeout)
+    def wait_for_inspection(self, timeout=None):
+        return self.client._poll(self, {'PENDING', 'UPLOADING', 'IMPORTING', 'CONVERTING', 'INSPECTING'}, timeout=timeout)
 
     def get_status(self):
         return self.describe().latest_dataset_version.status
