@@ -42,8 +42,8 @@ class Model():
     def list_versions(self):
         return self.client.list_model_versions(self.model_id)
 
-    def retrain(self):
-        return self.client.retrain_model(self.model_id)
+    def retrain(self, deployment_ids=[]):
+        return self.client.retrain_model(self.model_id, deployment_ids)
 
     def cancel_training(self):
         return self.client.cancel_model_training(self.model_id)
@@ -51,14 +51,14 @@ class Model():
     def delete(self):
         return self.client.delete_model(self.model_id)
 
-    def create_deployment(self, name=None, description=None, deployment_config=None):
-        return self.client.create_deployment(self.model_id, name, description, deployment_config)
+    def create_deployment(self, name=None, description=None, calls_per_second=None):
+        return self.client.create_deployment(self.model_id, name, description, calls_per_second)
 
     def wait_for_training(self, timeout=None):
         return self.client._poll(self, {'PENDING', 'TRAINING'}, delay=30, timeout=timeout)
 
     def wait_for_evaluation(self, timeout=None):
-        return self.client._poll(self, {'PENDING', 'TRAINING', 'EVALUATING'}, delay=30, timeout=timeout)
+        return self.wait_for_training()
 
     def get_status(self):
         return self.describe().latest_model_version.status
